module demo(in_clk, in_reset, in_enable, in_key, in_busy, 
				out_data, out_start, out_data_dac1, out_data_dac2, out_led);

input in_clk;
input in_reset;
input in_enable;
input in_busy;
input in_key;

output out_start;
output out_led;
output [15:0] out_data;
output [13:0] out_data_dac1;
output [13:0] out_data_dac2;

reg start;
reg [15:0] data;
reg [13:0] pattern_1;
reg [13:0] pattern_2;
reg [13:0] pattern_3; 
reg [5:0] counter_pattern;

reg key1;
reg key2;
reg busy1;
reg busy2;

reg [3:0] mode;
reg [3:0] counter_pac;
reg [15:0] timer_delay;
reg [13:0] counter;

reg led;
wire key;
wire busy;

assign out_start = start;
assign out_data = data;
assign out_data_dac1 = pattern_2;
assign out_data_dac2 = pattern_1;
assign key = key1 & key2;
assign busy = !busy1 & busy2;
assign out_led = led;

//key1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key1 <= 1'd1;
			end
		else
			begin
				if(in_enable == 1'd1) key1 <= in_key;
				else key1 <= 1'd1;
 			end
	end
	
//key2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) key2 <= ~key1;
				else key2 <= 1'd0;
			end
	end	
	
//busy1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy1 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy1 <= in_busy;
				else busy1 <= 1'd0;
			end
	end

//busy2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy2 <= busy1;
				else busy2 <= 1'd0;
			end
	end

//mode
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				mode <= 4'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(key == 1'd1 && mode < 4'd4) mode <= mode + 1;
						else if(mode == 4'd4) mode <= 4'd0;
					end
				else mode <= 4'd0;
			end
	end

//data
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				data <= 16'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(mode == 4'd0)
							begin
								if(counter_pac == 4'd0) data <= 16'h0104; 
								else if(counter_pac == 4'd1) data <= 16'h0280;  
								else if(counter_pac == 4'd2) data <= 16'h0303; 
								else if(counter_pac == 4'd3) data <= 16'h0480; 
							end
						else if(mode == 4'd1)
							begin
								if(counter_pac == 4'd0) data <= 16'h0144; 
								else if(counter_pac == 4'd1) data <= 16'h0280; 
								else if(counter_pac == 4'd2) data <=  16'h0302; 
								else if(counter_pac == 4'd3) data <= 16'h0480; 				
							end
						else if(mode == 4'd2)
							begin
								if(counter_pac == 4'd0) data <= 16'h0184; 
								else if(counter_pac == 4'd1) data <= 16'h0280; 
								else if(counter_pac == 4'd2) data <= 16'h0301; 
								else if(counter_pac == 4'd3) data <= 16'h0480;							
							end
						else if(mode == 4'd3)
							begin
								if(counter_pac == 4'd0) data <= 16'h01C4; 
								else if(counter_pac == 4'd1) data <= 16'h0280; 
								else if(counter_pac == 4'd2) data <= 16'h0300; 
								else if(counter_pac == 4'd3) data <= 16'h0480; 								
							end
					
					end
			end
	end
	
//counter_pac
always @(posedge in_clk)
		begin
			if(in_reset == 1'd0)
				begin
					counter_pac <= 4'd0;
				end
			else
				begin
					if(in_enable == 1'd1)
						begin
							if(key == 1'd1) counter_pac <= 4'd0;
							else if(busy == 1'd1 && counter_pac < 4'd4) counter_pac <= counter_pac + 1;
						end
					else counter_pac <= 4'd0;
				end
		end
		
//timer_delay
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				timer_delay <= 16'd0;
			end
		else
			begin
				if(start == 1'd0 && timer_delay < 16'd3500) timer_delay <= timer_delay + 1;
				else if(start == 1'd1) timer_delay <= 16'd0;
			end
	end
	
//start
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				start <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(timer_delay == 16'd3500 && counter_pac != 4'd4 && counter_pac != 4'd4) start <= 1'd1;
						else if(timer_delay == 16'd0 && in_busy == 1'd0)  start <= 1'd0;
					end
				else start <= 1'd0;
			end
	end
	

//counter_pattern

always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter_pattern <= 6'h0;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern < 6'd6) counter_pattern <= counter_pattern + 1;
						else if(counter_pattern == 6'd6) counter_pattern <= 6'h0;
					end
				else counter_pattern <= 6'h0;
			end
	end

//counter
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter <= 14'h00;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern == 6'h5 && counter < 14'd100) counter <= counter + 1;
						else if(counter == 14'd100) counter <= 14'd0;
					end
				else counter <= 14'h00;
			end
	end
				
	//pattern_1
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) pattern_1 <= 14'd0;
		else
			begin
				if(in_enable == 1'h1)
					begin
				
					 
				if(counter == 14'd0) pattern_1 <= 14'd7165; 
				else if(counter == 14'd1) pattern_1 <= 14'd7677; 
						else if(counter == 14'd2) pattern_1 <= 14'd8191; 
						else if(counter == 14'd3) pattern_1 <= 14'd8705; 
						else if(counter == 14'd4) pattern_1 <= 14'd9217; 
						else if(counter == 14'd5) pattern_1 <= 14'd9725; 
						else if(counter == 14'd6) pattern_1 <= 14'd10228; 
						else if(counter == 14'd7) pattern_1 <= 14'd10722; 
						else if(counter == 14'd8) pattern_1 <= 14'd11206; 
						else if(counter == 14'd9) pattern_1 <= 14'd11678; 
						else if(counter == 14'd10) pattern_1 <= 14'd12137; 
						else if(counter == 14'd11) pattern_1 <= 14'd12579; 
						else if(counter == 14'd12) pattern_1 <= 14'd13005; 
						else if(counter == 14'd13) pattern_1 <= 14'd13412; 
						else if(counter == 14'd14) pattern_1 <= 14'd13798; 
						else if(counter == 14'd15) pattern_1 <= 14'd14161; 
						else if(counter == 14'd16) pattern_1 <= 14'd14502; 
						else if(counter == 14'd17) pattern_1 <= 14'd14817; 
						else if(counter == 14'd18) pattern_1 <= 14'd15106; 
						else if(counter == 14'd19) pattern_1 <= 14'd15368; 
						else if(counter == 14'd20) pattern_1 <= 14'd15602; 
						else if(counter == 14'd21) pattern_1 <= 14'd15806; 
						else if(counter == 14'd22) pattern_1 <= 14'd15981; 
						else if(counter == 14'd23) pattern_1 <= 14'd16124; 
						else if(counter == 14'd24) pattern_1 <= 14'd16236; 
						else if(counter == 14'd25) pattern_1 <= 14'd16317; 
						else if(counter == 14'd26) pattern_1 <= 14'd16365; 
						else if(counter == 14'd27) pattern_1 <= 14'd16382; 
						else if(counter == 14'd28) pattern_1 <= 14'd16365; 
						else if(counter == 14'd29) pattern_1 <= 14'd16317; 
						else if(counter == 14'd30) pattern_1 <= 14'd16236; 
						else if(counter == 14'd31) pattern_1 <= 14'd16124; 
						else if(counter == 14'd32) pattern_1 <= 14'd15981; 
						else if(counter == 14'd33) pattern_1 <= 14'd15806; 
						else if(counter == 14'd34) pattern_1 <= 14'd15602; 
						else if(counter == 14'd35) pattern_1 <= 14'd15368; 
						else if(counter == 14'd36) pattern_1 <= 14'd15106; 
						else if(counter == 14'd37) pattern_1 <= 14'd14817; 
						else if(counter == 14'd38) pattern_1 <= 14'd14502; 
						else if(counter == 14'd39) pattern_1 <= 14'd14161; 
						else if(counter == 14'd40) pattern_1 <= 14'd13798; 
						else if(counter == 14'd41) pattern_1 <= 14'd13412; 
						else if(counter == 14'd42) pattern_1 <= 14'd13005; 
						else if(counter == 14'd43) pattern_1 <= 14'd12579; 
						else if(counter == 14'd44) pattern_1 <= 14'd12137; 
						else if(counter == 14'd45) pattern_1 <= 14'd11678; 
						else if(counter == 14'd46) pattern_1 <= 14'd11206; 
						else if(counter == 14'd47) pattern_1 <= 14'd10722; 
						else if(counter == 14'd48) pattern_1 <= 14'd10228; 
						else if(counter == 14'd49) pattern_1 <= 14'd9725; 
						else if(counter == 14'd50) pattern_1 <= 14'd9217; 
						else if(counter == 14'd51) pattern_1 <= 14'd8705; 
						else if(counter == 14'd52) pattern_1 <= 14'd8191; 
						else if(counter == 14'd53) pattern_1 <= 14'd7677; 
						else if(counter == 14'd54) pattern_1 <= 14'd7165; 
						else if(counter == 14'd55) pattern_1 <= 14'd6657; 
						else if(counter == 14'd56) pattern_1 <= 14'd6154; 
						else if(counter == 14'd57) pattern_1 <= 14'd5660; 
						else if(counter == 14'd58) pattern_1 <= 14'd5176; 
						else if(counter == 14'd59) pattern_1 <= 14'd4704; 
						else if(counter == 14'd60) pattern_1 <= 14'd4245; 
						else if(counter == 14'd61) pattern_1 <= 14'd3803; 
						else if(counter == 14'd62) pattern_1 <= 14'd3377; 
						else if(counter == 14'd63) pattern_1 <= 14'd2970; 
						else if(counter == 14'd64) pattern_1 <= 14'd2584; 
						else if(counter == 14'd65) pattern_1 <= 14'd2221; 
						else if(counter == 14'd66) pattern_1 <= 14'd1880; 
						else if(counter == 14'd67) pattern_1 <= 14'd1565; 
						else if(counter == 14'd68) pattern_1 <= 14'd1276; 
						else if(counter == 14'd69) pattern_1 <= 14'd1014; 
						else if(counter == 14'd70) pattern_1 <= 14'd780; 
						else if(counter == 14'd71) pattern_1 <= 14'd576; 
						else if(counter == 14'd72) pattern_1 <= 14'd401; 
						else if(counter == 14'd73) pattern_1 <= 14'd258; 
						else if(counter == 14'd74) pattern_1 <= 14'd146; 
						else if(counter == 14'd75) pattern_1 <= 14'd65; 
						else if(counter == 14'd76) pattern_1 <= 14'd17; 
						else if(counter == 14'd77) pattern_1 <= 14'd0;
						else if(counter == 14'd78) pattern_1 <= 14'd17; 
						else if(counter == 14'd79) pattern_1 <= 14'd65; 
						else if(counter == 14'd80) pattern_1 <= 14'd146; 
						else if(counter == 14'd81) pattern_1 <= 14'd258; 
						else if(counter == 14'd82) pattern_1 <= 14'd401; 
						else if(counter == 14'd83) pattern_1 <= 14'd576; 
						else if(counter == 14'd84) pattern_1 <= 14'd780; 
						else if(counter == 14'd85) pattern_1 <= 14'd1014; 
						else if(counter == 14'd86) pattern_1 <= 14'd1276; 
						else if(counter == 14'd87) pattern_1 <= 14'd1565; 
						else if(counter == 14'd88) pattern_1 <= 14'd1880; 
						else if(counter == 14'd89) pattern_1 <= 14'd2221; 
						else if(counter == 14'd90) pattern_1 <= 14'd2584; 
						else if(counter == 14'd91) pattern_1 <= 14'd2970; 
						else if(counter == 14'd92) pattern_1 <= 14'd3377; 
						else if(counter == 14'd93) pattern_1 <= 14'd3803;// 
						else if(counter == 14'd94) pattern_1 <= 14'd4245; 
						else if(counter == 14'd95) pattern_1 <= 14'd4704; 
						else if(counter == 14'd96) pattern_1 <= 14'd5176; 
						else if(counter == 14'd97) pattern_1 <= 14'd5660; 
						else if(counter == 14'd98) pattern_1 <= 14'd6154; 
						else if(counter == 14'd99) pattern_1 <= 14'd6657; 
					end
				else pattern_1 <= 14'd0;
			end
	end
		
//pattern_2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				pattern_2 <= 14'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
					if(counter == 8'd0) pattern_2 <= 14'd0; 
						else if(counter == 8'd1) pattern_2 <= 14'd163; 
						else if(counter == 8'd2) pattern_2 <= 14'd326; 
						else if(counter == 8'd3) pattern_2 <= 14'd489; 
						else if(counter == 8'd4) pattern_2 <= 14'd652; 
						else if(counter == 8'd5) pattern_2 <= 14'd815; 
						else if(counter == 8'd6) pattern_2 <= 14'd978; 
						else if(counter == 8'd7) pattern_2 <= 14'd1141; 
						else if(counter == 8'd8) pattern_2 <= 14'd1304; 
						else if(counter == 8'd9) pattern_2 <= 14'd1467; 
						else if(counter == 8'd10) pattern_2 <= 14'd1630; 
						else if(counter == 8'd11) pattern_2 <= 14'd1793; 
						else if(counter == 8'd12) pattern_2 <= 14'd1956; 
						else if(counter == 8'd13) pattern_2 <= 14'd2119; 
						else if(counter == 8'd14) pattern_2 <= 14'd2282; 
						else if(counter == 8'd15) pattern_2 <= 14'd2445; 
						else if(counter == 8'd16) pattern_2 <= 14'd2608; 
						else if(counter == 8'd17) pattern_2 <= 14'd2771; 
						else if(counter == 8'd18) pattern_2 <= 14'd2934; 
						else if(counter == 8'd19) pattern_2 <= 14'd3097; 
						else if(counter == 8'd20) pattern_2 <= 14'd3260; 
						else if(counter == 8'd21) pattern_2 <= 14'd3423; 
						else if(counter == 8'd22) pattern_2 <= 14'd3586; 
						else if(counter == 8'd23) pattern_2 <= 14'd3749; 
						else if(counter == 8'd24) pattern_2 <= 14'd3912; 
						else if(counter == 8'd25) pattern_2 <= 14'd4075; 
						else if(counter == 8'd26) pattern_2 <= 14'd4238; 
						else if(counter == 8'd27) pattern_2 <= 14'd4401; 
						else if(counter == 8'd28) pattern_2 <= 14'd4564; 
						else if(counter == 8'd29) pattern_2 <= 14'd4727; 
						else if(counter == 8'd30) pattern_2 <= 14'd4890; 
						else if(counter == 8'd31) pattern_2 <= 14'd5053; 
						else if(counter == 8'd32) pattern_2 <= 14'd5216; 
						else if(counter == 8'd33) pattern_2 <= 14'd5379; 
						else if(counter == 8'd34) pattern_2 <= 14'd5542; 
						else if(counter == 8'd35) pattern_2 <= 14'd5705; 
						else if(counter == 8'd36) pattern_2 <= 14'd5868; 
						else if(counter == 8'd37) pattern_2 <= 14'd6031; 
						else if(counter == 8'd38) pattern_2 <= 14'd6194; 
						else if(counter == 8'd39) pattern_2 <= 14'd6357; 
						else if(counter == 8'd40) pattern_2 <= 14'd6520; 
						else if(counter == 8'd41) pattern_2 <= 14'd6683; 
						else if(counter == 8'd42) pattern_2 <= 14'd6846; 
						else if(counter == 8'd43) pattern_2 <= 14'd7009; 
						else if(counter == 8'd44) pattern_2 <= 14'd7172; 
						else if(counter == 8'd45) pattern_2 <= 14'd7335; 
						else if(counter == 8'd46) pattern_2 <= 14'd7498; 
						else if(counter == 8'd47) pattern_2 <= 14'd7661; 
						else if(counter == 8'd48) pattern_2 <= 14'd7824; 
						else if(counter == 8'd49) pattern_2 <= 14'd7987; 
						else if(counter == 8'd50) pattern_2 <= 14'd8150; 
						else if(counter == 8'd51) pattern_2 <= 14'd8313; 
						else if(counter == 8'd52) pattern_2 <= 14'd8476; 
						else if(counter == 8'd53) pattern_2 <= 14'd8639; 
						else if(counter == 8'd54) pattern_2 <= 14'd8802; 
						else if(counter == 8'd55) pattern_2 <= 14'd8965; 
						else if(counter == 8'd56) pattern_2 <= 14'd9128; 
						else if(counter == 8'd57) pattern_2 <= 14'd9291; 
						else if(counter == 8'd58) pattern_2 <= 14'd9454; 
						else if(counter == 8'd59) pattern_2 <= 14'd9617; 
						else if(counter == 8'd60) pattern_2 <= 14'd9780; 
						else if(counter == 8'd61) pattern_2 <= 14'd9943; 
						else if(counter == 8'd62) pattern_2 <= 14'd10110; 
						else if(counter == 8'd63) pattern_2 <= 14'd10270; 
						else if(counter == 8'd64) pattern_2 <= 14'd10430; 
						else if(counter == 8'd65) pattern_2 <= 14'd10600; 
						else if(counter == 8'd66) pattern_2 <= 14'd10760; 
						else if(counter == 8'd67) pattern_2 <= 14'd10920; 
						else if(counter == 8'd68) pattern_2 <= 14'd11080; 
						else if(counter == 8'd69) pattern_2 <= 14'd11250; 
						else if(counter == 8'd70) pattern_2 <= 14'd11410; 
						else if(counter == 8'd71) pattern_2 <= 14'd11570; 
						else if(counter == 8'd72) pattern_2 <= 14'd11740; 
						else if(counter == 8'd73) pattern_2 <= 14'd11900; 
						else if(counter == 8'd74) pattern_2 <= 14'd12060; 
						else if(counter == 8'd75) pattern_2 <= 14'd12220; 
						else if(counter == 8'd76) pattern_2 <= 14'd12390; 
						else if(counter == 8'd77) pattern_2 <= 14'd12550; 
						else if(counter == 8'd78) pattern_2 <= 14'd12710; 
						else if(counter == 8'd79) pattern_2 <= 14'd12880; 
						else if(counter == 8'd80) pattern_2 <= 14'd13040; 
						else if(counter == 8'd81) pattern_2 <= 14'd13200; 
						else if(counter == 8'd82) pattern_2 <= 14'd13370; 
						else if(counter == 8'd83) pattern_2 <= 14'd13530; 
						else if(counter == 8'd84) pattern_2 <= 14'd13690; 
						else if(counter == 8'd85) pattern_2 <= 14'd13850; 
						else if(counter == 8'd86) pattern_2 <= 14'd14020; 
						else if(counter == 8'd87) pattern_2 <= 14'd14180; 
						else if(counter == 8'd88) pattern_2 <= 14'd14340; 
						else if(counter == 8'd89) pattern_2 <= 14'd14510; 
						else if(counter == 8'd90) pattern_2 <= 14'd14670; 
						else if(counter == 8'd91) pattern_2 <= 14'd14830; 
						else if(counter == 8'd92) pattern_2 <= 14'd15000; 
						else if(counter == 8'd93) pattern_2 <= 14'd15160; 
						else if(counter == 8'd94) pattern_2 <= 14'd15320; 
						else if(counter == 8'd95) pattern_2 <= 14'd15480; 
						else if(counter == 8'd96) pattern_2 <= 14'd15650; 
						else if(counter == 8'd97) pattern_2 <= 14'd15810; 
						else if(counter == 8'd98) pattern_2 <= 14'd15970; 
						else if(counter == 8'd99) pattern_2 <= 14'd16140;			
					end
				else pattern_2 <= 14'd0;					
			end
	end
	
//led
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				led <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1 ) led <= 1'd1;
				else led <= 1'd0;
			end
			
	end
	
endmodule