module nucleus(in_clk, in_reset, in_sclk_arm, in_mosi_arm, in_ss_arm, in_sdo_dac, in_clk_160,
					out_miso_arm, out_sclk_dac, out_sdio_dac, out_ss_dac, out_enable_demo, out_pattern1, out_pattern2, out_clk_ram, out_rden, out_wren, out_addr, out_addr_buf, out_test);

input in_clk;
input in_clk_160;
input in_reset;
input in_sclk_arm;
input in_mosi_arm;
input in_ss_arm;
input in_sdo_dac;

output out_miso_arm;
output out_sclk_dac;
output out_sdio_dac;
output out_ss_dac;
output out_enable_demo;
output out_test;
output out_clk_ram;
output out_wren;
output out_rden;

output [13:0] out_pattern1;
output [13:0] out_pattern2;
output [12:0] out_addr;
output [7:0] out_addr_buf;
	
reg enable_demo;
reg control_data;

reg [7:0] counter_in_data;
reg [7:0] counter_in_pattern;
reg [7:0] counter_clk_ram;
reg [15:0] control_buf;
reg [31:0] data_buf;
reg [7:0] div_buf;
reg [13:0] pattern1_buf;
reg [13:0] pattern2_buf;
reg [12:0] addr_w;
reg [12:0] addr_r;
reg [12:0] addr_buf;

reg wren;
reg rden;
reg clk_ram_wren;
reg clk_ram_rden;
reg sclk_buf1;
reg sclk_buf2;
reg wren_buf;
reg wren_buf1;
reg wren_buf2;


wire sclk;
wire sclk_neg;
wire wren_neg;
wire clk_rden;

assign wren_neg = !wren & wren_buf;
assign wren_pos3 = !wren_buf2 & wren_buf1 & wren_buf2;
assign sclk = !sclk_buf2 & sclk_buf1;
assign sclk_neg = !sclk_buf1 * sclk_buf2;
assign out_enable_demo = enable_demo;
assign clk_rden = (div_buf != 8'd0 && div_buf != 8'd1) ? clk_ram_rden : in_clk_160;
assign out_clk_ram = (control_data) ? clk_ram_wren : clk_rden;
assign out_rden = rden;
assign out_wren = wren;
assign out_addr = (control_data) ? addr_w : addr_r;
assign out_pattern1 = pattern1_buf;
assign out_pattern2 = pattern2_buf;
assign out_sclk_dac = in_sclk_arm;
assign out_sdio_dac = in_mosi_arm;
assign out_ss_dac = in_ss_arm;
assign out_miso_arm = in_sdo_dac;
assign out_addr_buf[7:0] = control_buf[15:8];
assign out_control_data = enable_demo;

//sclk_buf1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) sclk_buf1 <= 1'd1;
		else
			begin
				sclk_buf1 <= in_sclk_arm;
			end
	end

//sclk_buf2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) sclk_buf2 <= 1'd1;
		else
			begin
				sclk_buf2 <= sclk_buf1;
			end
	end

//enable_demo
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) enable_demo <= 1'd1;
		else
			begin
				if(in_ss_arm == 1'd0) enable_demo <= 1'd0;
			end
			
	end
	
//counter_in_data
always @(negedge in_clk)
	begin
		if(in_reset == 1'd0) counter_in_data <= 8'd0;
		else
			begin
				if(in_ss_arm == 1'd0)	
					begin
						if(sclk == 1'd1 && counter_in_data < 8'd16) counter_in_data <= counter_in_data + 1;
					end
				else counter_in_data <= 8'd0;
			end
	end
	
//control_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) control_buf <= 16'd0;
		else 
			begin
				if(in_ss_arm == 1'd0) 
					begin
						if(counter_in_data < 8'd16 && sclk == 1'd1) control_buf[15:0] <= {control_buf[14:0], in_mosi_arm};
					end
				else control_buf <= 1'd0;
			end
	end

//div_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) div_buf <= 8'd0;
		else
			begin
				if(control_buf[15:8] == 8'd16) div_buf[7:0] <= control_buf[7:0];
			end
			
	end

//control_data 
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)  control_data <= 1'd0;
		else
			begin
				if(control_buf[15:8] == 8'd16 && counter_in_data == 8'd16) control_data <= 1'd1;
				else control_data <= 1'd0;
			end
	end

//counter_in_pattern
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) counter_in_pattern <= 8'd0;
		else
			begin
				if(counter_in_data == 8'd16 && control_buf [15:8] == 8'd16 && in_ss_arm ==1'd0)
					begin 
						if(counter_in_pattern < 8'd33 && sclk == 1'd1) counter_in_pattern <= counter_in_pattern + 1;
						else if(counter_in_pattern == 8'd33) counter_in_pattern <= 8'd1;
					end
				else counter_in_pattern <= 8'd0;
			end
	end
		
//data_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) data_buf <= 32'd0;
		else
			begin
				if(in_ss_arm == 1'd0 && control_data == 1'd1)
					begin
						if(sclk == 1'd1) data_buf[31:0] <= {data_buf[30:0],in_mosi_arm};
					end
			end
	end

//pattern1_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) pattern1_buf <= 14'd0;
		else
			begin
				if(control_data == 1'd1 && counter_in_pattern == 8'd32)
					begin
					pattern1_buf[7:0] <= data_buf[31:24];
					pattern1_buf[13:8] <= data_buf[21:16];
					end
			end
	end

//pattern2_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) pattern2_buf <= 14'd0;
		else
			begin
				if(control_data == 1'd1 && counter_in_pattern == 8'd32)
					begin
					pattern2_buf[7:0] <= data_buf[15:8];
					pattern2_buf[13:8] <= data_buf[5:0];
					end
			end
	end

//wren
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) wren <= 1'd0;
		else
			begin
				if(control_data == 1'd1 && counter_in_pattern == 8'd32)
					begin
						if(pattern1_buf[7:0] == data_buf[31:24] && pattern1_buf[13:8] == data_buf[21:16] && pattern2_buf[7:0] == data_buf[15:8] && pattern2_buf[13:8] <= data_buf[5:0]) wren <= 1'd1;
					end
				else wren <= 1'd0;
			end
	end

//wren_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) wren_buf <= 1'd0;
		else wren_buf <= wren;
	end
//wren_buf1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) wren_buf1 <= 1'd0;
		else wren_buf <= wren_buf;
	end
//wren_buf2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) wren_buf2 <= 1'd0;
		else wren_buf2 <= wren_buf1;
	end

//addr_w
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) addr_w <= 13'd0;
		else
			begin
				if(control_data == 1'd1)
					begin
						if(wren_neg == 1'd1 && addr_w < 13'd8192) addr_w  <= addr_w + 1;
						else if(addr_w == 13'd8192) addr_w <= 13'd0;
					end
				else addr_w <= 13'd0;
			end
	end
	
//addr_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) addr_buf <= 13'd0;
		else
			begin
				if(control_data == 1'd1 && wren == 1'd1) addr_buf <= addr_w;
			end
	end

//clk_ram_wren
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) clk_ram_wren <= 1'd0;
		else
			begin
				if(control_data == 1'd1)
					begin
						if(wren_pos3 == 1'd1) clk_ram_wren <= 1'd1;
						else if(wren == 1'd0) clk_ram_wren <= 1'd0;
					end
				else clk_ram_wren <= 1'd0;
			end
	end
	
//rden
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) rden <= 1'd0;
		else
			begin
				if(control_data == 1'd0 && enable_demo == 1'd0) rden <= 1'd1;
				else rden <= 1'd0;
			end
	end

//counter_clk_ram
always @(posedge in_clk_160)
	begin
		if(in_reset == 1'd0) counter_clk_ram <= 8'd0;
		else
			begin
				if(enable_demo == 1'd0 && rden == 1'd1)
					begin
						if(div_buf != 8'd0 && div_buf != 8'd1 && counter_clk_ram < div_buf - 1) counter_clk_ram <= counter_clk_ram + 1;
						else counter_clk_ram <= 8'd0;
					end
			end
	end
	
//clk_ram_rden
always @(posedge in_clk_160)
	begin
		if(in_reset == 1'd0) clk_ram_rden <= 1'd0;
		else
			begin
				if(enable_demo == 1'd0)
					begin
						if(counter_clk_ram == 8'd0) clk_ram_rden <= ~clk_ram_rden;
					end
				else clk_ram_rden <= 1'd0;
			end
	end

//addr_r
always @(negedge in_clk_160)
	begin
		if(in_reset == 1'd0) addr_r <= 13'd0;
		else
			begin
				if(control_data == 1'd0 && enable_demo == 1'd0)
					begin
						if(div_buf != 8'd0 && div_buf != 8'd1)
							begin
								if(counter_clk_ram == 8'd0 && addr_r <= addr_buf) addr_r <= addr_r + 1;
								else if(addr_r > addr_buf) addr_r <= 13'd0;
							end
						else 
							begin
								if(addr_r <= addr_buf) addr_r <= addr_r + 1;
								else addr_r <= 13'd0;
							end
					end
				else addr_r <= 13'd0;
			end
	end

endmodule
